//=============================================================================
// NALEInventory.js
//=============================================================================
/*:
 * @plugindesc The Evidence Screen as specified by these requirements for Nina Aquila: Leagle Eagle
 * https://www.reddit.com/r/RPGMaker/comments/a69szb/mvpaid_need_a_scripter_to_create_custom_inventory/
 *
 * @author Tom Maenan Read Cutting
 *
 * @help
 *
 * Ask me if anything is unclear!
 *
 * @param Present Title
 * @desc The text contained in the title window of the present screen.
 * Default: PRESENT
 * @default PRESENT
 * @require 1
 * @type string
 *
 * @param Evidence Title
 * @desc The text contained in the title window of the evidence screen.
 * Default: EVIDENCE
 * @default EVIDENCE
 * @require 1
 * @type string
 *
 * @param Title Text Width
 * @default 100
 * @type number
 *
 * @param Title Text Height
 * @default 25
 * @type number
 *
 * @param Title Text Font Size
 * @default 28
 * @type number
 *
 *
 * @param Instruction Text
 * @desc The text contained in the title window of the evidence screen.
 * Default: Examine your evidence. Hit <cancel> (B button/X Key/ Two-Finger-Tap) to go back.
 * @default Examine your evidence. Hit <cancel> (B button/X Key/ Two-Finger-Tap) to go back.
 * @require 1
 * @type string
 *
 * @param Instruction Text Present
 * @desc The text contained in the title window of the present screen.
 * Default: Examine your evidence. Hit <cancel> (B button/X Key/ Two-Finger-Tap) to go back.
 * @default Examine your evidence. Hit <cancel> (B button/X Key/ Two-Finger-Tap) to go back.
 * @require 1
 * @type string
 *
 * @param Instruction Text Width
 * @default 400
 * @type number
 *
 * @param Instruction Text Height
 * @default 50
 * @type number
 *
 * @param Instruction Text Font Size
 * @default 28
 * @type number
 *
 * @param Horizontal Divider Position
 * @desc A value between 0.1 and 0.9 determining where the horizontal divider is put.
 * @default 0.33
 * @min 0.1
 * @max 0.9
 * @type number
 *
 * @requiredAssets img/items/placeholder
 *
 */

(function() {

// Useful documentation
//
// Item Descriptions:
// https://kinoar.github.io/rmmv-doc-web/classes/window_help.html
//
// Item List:
// https://kinoar.github.io/rmmv-doc-web/classes/window_itemlist.html
//
// Item Scene:
// https://kinoar.github.io/rmmv-doc-web/classes/scene_item.html

function LoadPlaceholderBitmap()
{
    return ImageManager.loadBitmap("img/items/", "placeholder");
}

// function LoadItemBitmap(itemNumber)
// {
//     var item_image = ImageManager.loadBitmap("img/items/", itemNumber);
//     return new Sprite(item_image);
// }

// TODO: Disable dev code for release builds.
function LoadItemBitmap(itemNumber)
{
    var itemNumberString = "" + itemNumber;
    var pad = "0000";
    itemNumberString = pad.substring(0, pad.length - itemNumberString.length) + itemNumberString;
    return ImageManager.loadBitmap("img/items/", itemNumberString);
}

var parameters = PluginManager.parameters('NALEInventory');
var evidenceTitle = String(parameters['Evidence Title']);
var presentTitle = String(parameters['Present Title']);
var titleTextWidth = Number(parameters['Title Text Width']);
var titleTextHeight = Number(parameters['Title Text Height']);
var titleTextFontSize = Number(parameters['Title Text Font Size']);
var instructionWindowText = String(parameters['Instruction Text']);
var instructionPresentWindowText = String(parameters['Instruction Text Present']);
var instructionTextWidth = Number(parameters['Instruction Text Width']);
var instructionTextHeight = Number(parameters['Instruction Text Height']);
var instructionTextFontSize = Number(parameters['Instruction Text Font Size']);
var horizonalDivider = Number(parameters['Horizontal Divider Position']);

// ---- Define all of the window classes
function Window_Title() {
    this.initialize.apply(this, arguments);
};
Window_Title.prototype = Object.create(Window_Base.prototype);
Window_Title.prototype.constructor = Window_Title;
Window_Title.prototype.initialize = function(x, y, width, height, titleWindowText) {
    Window_Base.prototype.initialize.call(this, x, y, width, height);
    this._titleText = new Bitmap(titleTextWidth, titleTextHeight);
    this._titleText.fontSize = titleTextFontSize;
    this._titleText.drawText(titleWindowText, 0, this._titleText.height/2, this._titleText.width, 1, "center");
    this._titleTextSprite = new Sprite(this._titleText);
    this.addChild(this._titleTextSprite);
};
Window_Title.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    this._titleTextSprite.x = (this.width / 2) - (this._titleTextSprite.width / 2);
    this._titleTextSprite.y = (this.height / 2) - (this._titleTextSprite.height / 2);
};

function Window_Instruction() {
    this.initialize.apply(this, arguments);
};
Window_Instruction.prototype = Object.create(Window_Base.prototype);
Window_Instruction.prototype.constructor = Window_Instruction;
Window_Instruction.prototype.initialize = function(x, y, width, height, instructionTextContent) {
    Window_Base.prototype.initialize.call(this, x, y, width, height);
    this._instructionText = new Bitmap(instructionTextWidth, instructionTextHeight);
    this._instructionText.fontSize = instructionTextFontSize;
    this._instructionText.drawText(instructionTextContent, 0, this._instructionText.height/2, this._instructionText.width, 1, "center");
    this._instructionTextSprite = new Sprite(this._instructionText);
    this.addChild(this._instructionTextSprite);
};
Window_Instruction.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    this._instructionTextSprite.x = (this.width / 2) - (this._instructionTextSprite.width / 2);
    this._instructionTextSprite.y = (this.height / 2) - (this._instructionTextSprite.height / 2);
};

function Window_Category() {
    this.initialize.apply(this, arguments);
};
Window_Category.prototype = Object.create(Window_Base.prototype);
Window_Category.prototype.constructor = Window_Category;
Window_Category.prototype.initialize = function(x, y, width, height) {
    Window_Base.prototype.initialize.call(this, x, y, width, height);
};

function Window_Picture() {
    this.initialize.apply(this, arguments);
};
Window_Picture.prototype = Object.create(Window_Base.prototype);
Window_Picture.prototype.constructor = Window_Picture;
Window_Picture.prototype.initialize = function(x, y, width, height, itemList) {
    Window_Base.prototype.initialize.call(this, x, y, width, height);
    this._itemList = itemList;
    this._selectedItem = 1;
    this._sprite = new Sprite(LoadItemBitmap(this._selectedItem));
    this.addChild(this._sprite);
};
Window_Picture.prototype.update = function() {
    Window_Base.prototype.update.call(this);

    if(this._itemList.item() != null && this._selectedItem != this._itemList.item().id)
    {
        this._selectedItem = this._itemList.item().id;

        var bitmapItem = LoadItemBitmap(this._selectedItem);
        this._sprite.bitmap = bitmapItem;
    }
    this.configureSprite();
};
Window_Picture.prototype.configureSprite = function() {
    this._sprite.x = (this.width / 2) - (this._sprite.width / 2);
    this._sprite.y = (this.height / 2) - (this._sprite.height / 2);
};

// ---- Setup the Item List Window

Window_ItemList.prototype.maxCols = function() {
    return 1;
};

Window_ItemList_update = Window_ItemList.prototype.update
Window_ItemList.prototype.update = function() {
    Window_ItemList_update.call(this);
}
Window_ItemList_makeItemList = Window_ItemList.prototype.makeItemList
Window_ItemList.prototype.makeItemList = function() {
    Window_ItemList_makeItemList.call(this);
    if(this._data.length > 0) {
        this._data.sort(function(item1, item2) {
            if(item1.note.includes("[hide]")) { return true; }
            if(item2.note.includes("[hide]")) { return false; }
            var topString = "[top]";
            if(item1.note.includes(topString) && item2.note.includes(topString)) {
                return item1.name > item2.name;
            } else if (item1.note.includes(topString)) {
                return false;
            } else if (item2.note.includes(topString)) {
                return true;
            } else {
                return item1.name > item2.name;
            }
        });
        while(this._data[this._data.length - 1].note.includes("[hide]"))
        {
            this._data.pop();
        }
    }
}
Window_ItemList_drawItem = Window_ItemList.prototype.drawItem
Window_ItemList.prototype.drawItem = function(index) {
    if(!this._data[index].note.includes("[hide]")) {
        Window_ItemList_drawItem.call(this, index);
    }
}

// ---- Setup the EVIDENCE screen

Scene_Item_create = Scene_Item.prototype.create
Scene_Item.prototype.create = function() {
    Scene_Item_create.call(this);

    this._titleWindow = new Window_Title(0, 0, 0, 0, evidenceTitle);
    this._instructionWindow = new Window_Instruction(0, 0, 0, 0, instructionWindowText);
    this._itemListWindow = this._itemWindow;
    this._categoryWindow = new Window_Category(0, 0, 0, 0);
    this._pictureWindow = new Window_Picture(0, 0, 0, 0, this._itemListWindow);
    this._helpWindow = this._helpWindow;

    this.addChild(this._titleWindow);
    this.addChild(this._instructionWindow);
    this.addChild(this._categoryWindow);
    this.addChild(this._pictureWindow);


    // ---- Calculate various windows layout information
    this._titleWindow.width = Graphics.boxWidth * horizonalDivider;
    this._titleWindow.height = Graphics.boxHeight * 0.20;

    this._instructionWindow.x = this._titleWindow.width;
    this._instructionWindow.width = Graphics.boxWidth - this._titleWindow.width;
    this._instructionWindow.height = this._titleWindow.height;

    this._itemListWindow.y = this._titleWindow.height;
    this._itemListWindow.width = this._titleWindow.width;
    this._itemListWindow.height = Graphics.boxHeight - this._titleWindow.height;


    this._pictureWindow.x = this._itemListWindow.width;
    this._pictureWindow.y = this._instructionWindow.height;
    this._pictureWindow.width = Graphics.boxWidth - this._itemListWindow.width;
    this._pictureWindow.height = this._itemListWindow.height * 0.75;

    this._helpWindow.x = this._itemListWindow.width;
    this._helpWindow.y = this._instructionWindow.height + this._pictureWindow.height;
    this._helpWindow.width = this._pictureWindow.width;
    this._helpWindow.height = this._itemListWindow.height - this._pictureWindow.height;
    // ---- End of layout maths
}

// ---- End evidence screen
Scene_Menu_createCommandWindow = Scene_Menu.prototype.createCommandWindow
Scene_Menu.prototype.createCommandWindow = function() {
    Scene_Menu_createCommandWindow.call(this);
    this._commandWindow.setHandler('Select Item', function() { console.log("here") });
}

// ---- Present screen, many shennanigans to get this working :(
Window_EventItem_create = Window_EventItem.prototype.initialize
Window_EventItem.prototype.initialize = function(messageWindow) {
    Window_EventItem_create.call(this, messageWindow);


    this._titleWindow = new Window_Title(0, 0, 0, 0, presentTitle);
    this._instructionWindow = new Window_Instruction(0, 0, 0, 0, instructionPresentWindowText);
    this._itemListWindow = this;
    this._categoryWindow = new Window_Category(0, 0, 0, 0);
    this._pictureWindow = new Window_Picture(0, 0, 0, 0, this._itemListWindow);
    this._helpWindow = new Window_Help();

    var windowskin = ImageManager.loadBitmap("img/system/", "WindowPresent");
    this._titleWindow.windowskin = windowskin;
    this._instructionWindow.windowskin = windowskin;
    this._itemListWindow.windowskin = windowskin;
    this._categoryWindow.windowskin = windowskin;
    this._pictureWindow.windowskin = windowskin;
    this._helpWindow.windowskin = windowskin;

    this.addChild(this._titleWindow);
    this.addChild(this._instructionWindow);
    this.addChild(this._categoryWindow);
    this.addChild(this._pictureWindow);
    this.addChild(this._helpWindow);

    var totalHeight = Graphics.boxHeight * 0.75;

    // ---- Calculate various windows layout information
    this._titleWindow.width = Graphics.boxWidth * horizonalDivider;
    this._titleWindow.height = totalHeight * 0.20;
    this._titleWindow.y = 0 - this._titleWindow.height;

    this._instructionWindow.x = this._titleWindow.width;
    this._instructionWindow.y = 0 - this._titleWindow.height;
    this._instructionWindow.width = Graphics.boxWidth - this._titleWindow.width;
    this._instructionWindow.height = this._titleWindow.height;

    this._itemListWindow.y = this._titleWindow.height;
    this._itemListWindow.width = this._titleWindow.width;
    this._itemListWindow.height = totalHeight - this._titleWindow.height;

    this._pictureWindow.x = this._itemListWindow.width;
    this._pictureWindow.y = 0;
    this._pictureWindow.width = Graphics.boxWidth - this._itemListWindow.width;
    this._pictureWindow.height = this._itemListWindow.height / 1.5;

    this._helpWindow.x = this._itemListWindow.width;
    this._helpWindow.y = (this._instructionWindow.height + this._pictureWindow.height) - this._titleWindow.height;
    this._helpWindow.width = this._pictureWindow.width;
    this._helpWindow.height = this._itemListWindow.height - this._pictureWindow.height;
    // ---- End of layout maths
}

// Disable this, as we want to position the event_item screen :)
Window_EventItem.prototype.updatePlacement = function() {};

})(); // end opening function
